@@ -19,6 +19,8 @@ for f in os.listdir(dirname):
 
 from safe.impact_functions.core import FunctionProvider
 from safe.impact_functions.core import get_plugins  # FIXME: Deprecate
+from safe.impact_functions.core import get_plugin
 from safe.impact_functions.core import get_admissible_plugins
 from safe.impact_functions.core import compatible_layers
 from safe.impact_functions.core import get_function_title
+from safe.impact_functions.core import get_documentation
